from dataclasses import dataclass
import re
from PIL import Image, ImageDraw, ImageFont
from types import SimpleNamespace

@dataclass
class BaseColor:
    hex: str = None
    rgb: tuple[int, int, int] = None
    cmyk: tuple[int, int, int, int] = None

    def __post_init__(self):
        """Automatically complete missing color formats"""
        if self.hex:
            self.rgb = self.hex_to_rgb(self.hex)
            self.cmyk = self.rgb_to_cmyk(self.rgb)
        elif self.rgb:
            self.hex = self.rgb_to_hex(self.rgb)
            self.cmyk = self.rgb_to_cmyk(self.rgb)
        elif self.cmyk:
            self.rgb = self.cmyk_to_rgb(self.cmyk)
            self.hex = self.rgb_to_hex(self.rgb)
        else:
            raise ValueError("At least one of HEX, RGB, or CMYK must be provided")

    @staticmethod
    def hex_to_rgb(hex_color: str) -> tuple[int, int, int]:
        """Convert HEX to RGB"""
        hex_color = hex_color.lstrip("#")
        if re.fullmatch(r"[0-9A-Fa-f]{6}", hex_color):
            return tuple(int(hex_color[i:i+2], 16) for i in (0, 2, 4))
        raise ValueError("Invalid HEX color format")

    @staticmethod
    def rgb_to_hex(rgb_color: tuple[int, int, int]) -> str:
        """Convert RGB to HEX"""
        if not all(0 <= c <= 255 for c in rgb_color):
            raise ValueError("RGB values must be in the range 0-255")
        return "#{:02X}{:02X}{:02X}".format(*rgb_color)

    @staticmethod
    def rgb_to_cmyk(rgb_color: tuple[int, int, int]) -> tuple[int, int, int, int]:
        """Convert RGB to CMYK"""
        r, g, b = [x / 255.0 for x in rgb_color]
        k = 1 - max(r, g, b)
        if k == 1:
            return 0, 0, 0, 100  # Pure black
        c = (1 - r - k) / (1 - k) * 100
        m = (1 - g - k) / (1 - k) * 100
        y = (1 - b - k) / (1 - k) * 100
        k *= 100
        return int(c), int(m), int(y), int(k)

    @staticmethod
    def cmyk_to_rgb(cmyk_color: tuple[int, int, int, int]) -> tuple[int, int, int]:
        """Convert CMYK to RGB"""
        if not all(0 <= c <= 100 for c in cmyk_color):
            raise ValueError("CMYK values must be in the range 0-100")
        c, m, y, k = [x / 100.0 for x in cmyk_color]
        r = 255 * (1 - c) * (1 - k)
        g = 255 * (1 - m) * (1 - k)
        b = 255 * (1 - y) * (1 - k)
        return int(r), int(g), int(b)
    
    def canvas(self):
        """Display the color with a side color bar and annotated text"""
        width, height = 300, 100  # Canvas size
        color_bar_width = 100  # Width of the color bar

        # Create a blank white image
        canvas = Image.new("RGB", (width, height), "white")
        draw = ImageDraw.Draw(canvas)

        # Draw the color bar
        draw.rectangle([(width - color_bar_width, 0), (width, height)], fill=self.rgb)

        # Prepare text information
        text_lines = [
            f"HEX: {self.hex}",
            f"RGB: {self.rgb}",
            f"CMYK: {self.cmyk}"
        ]
        
        # Load a default font
        try:
            font = ImageFont.truetype("arial.ttf", 16)
        except IOError:
            font = ImageFont.load_default()

        # Draw text on the left side
        text_x, text_y = 10, 20
        for line in text_lines:
            draw.text((text_x, text_y), line, fill="black", font=font)
            text_y += 30  # Spacing between lines
        return canvas
    
    def show(self):
        """Display the color and information"""
        self.canvas().show()

class Color(SimpleNamespace):
    RED = BaseColor(hex="#FF0000")         # 红色
    GREEN = BaseColor(hex="#00FF00")       # 绿色
    BLUE = BaseColor(hex="#0000FF")        # 蓝色
    YELLOW = BaseColor(hex="#FFFF00")      # 黄色
    CYAN = BaseColor(hex="#00FFFF")        # 青色
    MAGENTA = BaseColor(hex="#FF00FF")     # 品红
    WHITE = BaseColor(hex="#FFFFFF")       # 白色
    BLACK = BaseColor(hex="#000000")       # 黑色
    GRAY = BaseColor(hex="#808080")        # 灰色
    ORANGE = BaseColor(hex="#FFA500")      # 橙色
    PURPLE = BaseColor(hex="#800080")      # 紫色
    PINK = BaseColor(hex="#FFC0CB")        # 粉色
    BROWN = BaseColor(hex="#A52A2A")       # 棕色
    
    PRUSSIAN_BLUE = BaseColor(hex="#003153")  # 普鲁士蓝色
    KLEIN_BLUE = BaseColor(hex="#002FA7")     # 克莱因蓝色
    MARRS_GREEN = BaseColor(hex="#01847F")    # 马尔斯绿色
    SCHONBRUNNER_GELB = BaseColor(hex="#FBD26A")  # 申布伦黄色
    BURGUNDY_RED = BaseColor(hex="#470024")  # 勃艮第红色
    
    
if __name__ == "__main__":    
    color = Color.MARRS_GREEN
    color.show()





from enum import Enum

class PantoneColor(Enum):
    CERULEAN = 2000  # 蔚蓝色, Pantone 15-4020
    FUCHSIA_ROSE = 2001  # 紫红玫瑰, Pantone 17-2031
    TRUE_RED = 2002  # 正红, Pantone 19-1664
    AQUA_SKY = 2003  # 水色天空, Pantone 14-4811
    TIGERLILY = 2004  # 虎皮百合, Pantone 17-1456
    BLUE_TURQUOISE = 2005  # 蓝绿松石, Pantone 15-5217
    SAND_DOLLAR = 2006  # 沙金色, Pantone 13-1106
    CHILI_PEPPER = 2007  # 辣椒红, Pantone 19-1557
    BLUE_IRIS = 2008  # 鸢尾蓝, Pantone 18-3943
    MIMOSA = 2009  # 含羞草黄, Pantone 14-0848
    TURQUOISE = 2010  # 松石绿, Pantone 15-5519
    HONEYSUCKLE = 2011  # 忍冬红, Pantone 18-2120
    TANGERINE_TANGO = 2012  # 探戈橘, Pantone 17-1463
    EMERALD = 2013  # 翡翠绿, Pantone 17-5641
    RADIANT_ORCHID = 2014  # 璀璨紫兰花, Pantone 18-3224
    MARSALA = 2015  # 玛萨拉酒红, Pantone 18-1438
    ROSE_QUARTZ = 2016  # 玫瑰石英粉红, Pantone 13-1520
    SERENITY = 2016  # 宁静粉蓝, Pantone 15-3919
    GREENERY = 2017  # 草木绿, Pantone 15-0343
    ULTRA_VIOLET = 2018  # 紫外光色, Pantone 18-3838
    LIVING_CORAL = 2019  # 珊瑚橘, Pantone 16-1546
    CLASSIC_BLUE = 2020  # 经典蓝, Pantone 19-4052
    ULTIMATE_GRAY = 2021  # 极致灰, Pantone 17-5104
    ILLUMINATING = 2021  # 亮丽黄, Pantone 13-0647
    VERY_PERI = 2022  # 长春花蓝, Pantone 17-3938
    VIVA_MAGENTA = 2023  # 非凡洋红, Pantone 18-1750
    PEACH_FUZZ = 2024  # 柔和桃, Pantone 13-1023

# 示例用法
print(PantoneColor.CLASSIC_BLUE.name, PantoneColor.CLASSIC_BLUE.value)



from io import BytesIO
import PIL
from datetime import datetime
import os
import sys
from typing import Optional, Union
from pathlib import Path
import argparse
import numpy as np
import io
import matplotlib.pyplot as plt
import base64
import yaml
import json

def uint8_imread(path: str):
    img = plt.imread(path)
    if img.dtype != np.uint8 and img.max() <= 1.0:
        img = img * 255
        img = img.astype(np.uint8)
    return img


def pil2str(x):
    buffer = BytesIO()
    x.save(buffer, format='PNG')
    b64 = base64.b64encode(buffer.getvalue())
    res = str(b64, 'utf-8')
    return res


def str2pil(s):
    b64 = base64.b64decode(s.encode('utf-8'))
    return PIL.Image.open(BytesIO(b64))


def get_datetime():
    return datetime.now().strftime("%Y%m%d-%H%M%S")


def init_project_dir_with(project_dir="."):
    def f(*args, **kwargs):
        return os.path.join(project_dir, *args, **kwargs)
    return f


def is_abs_path(path: Union[Path, str]):
    if isinstance(path, Path):
        return path.is_absolute()
    elif isinstance(path, str):
        return os.path.isabs(path)
    else:
        raise NotImplementedError 
        return False


def load_yaml(path):
    return yaml.safe_load(open(path, 'r'))


def load_namespace_from_yaml(path):
    D = load_yaml(path)
    return argparse.Namespace(**D)


def create_namespace(D:dict):
    return argparse.Namespace(**D)


def pt_to_pil(images):
    """
    Convert a torch image to a PIL image.
    """
    images = (images / 2 + 0.5).clamp(0, 1)
    images = images.cpu().permute(0, 2, 3, 1).float().numpy()
    images = numpy_to_pil(images)
    return images


def numpy_to_pil(np_images):
    """
    Convert a numpy image or a batch of images to PIL images.

    Args:
        np_images (numpy.ndarray): A numpy array of shape (H, W, C), (H, W), 
                                    or (N, H, W, C), (N, H, W).

    Returns:
        list: A list of PIL.Image objects, even if the input is a single image.
    """
    # Ensure batch dimension for single images
    if np_images.ndim == 3:  # Single RGB image or grayscale image (H, W, C)
        np_images = np_images[None, ...]
    elif np_images.ndim == 2:  # Single grayscale image (H, W)
        np_images = np_images[None, ..., None]  # Add channel and batch dimension

    # Ensure images are in uint8 format
    if np_images.dtype != np.uint8:
        np_images = (np_images * 255).round().astype("uint8")

    # Convert each image in the batch to a PIL Image
    pil_images = []
    for image in np_images:
        if image.shape[-1] == 1:  # Grayscale image
            pil_images.append(PIL.Image.fromarray(image.squeeze(), mode="L"))
        elif image.shape[-1] == 3:  # RGB image
            pil_images.append(PIL.Image.fromarray(image, mode="RGB"))
        elif image.shape[-1] == 4:  # RGBA image
            pil_images.append(PIL.Image.fromarray(image, mode="RGBA"))
        else:
            raise ValueError(f"Unsupported number of channels: {image.shape[-1]}")

    return pil_images if len(pil_images) > 1 else pil_images[0]  # Return a single image if applicable


def is_video_file(file_name):
    video_extensions = ['.mp4', '.avi', '.mov', '.mkv', '.wmv']  # 添加其他视频文件格式
    return any(file_name.lower().endswith(ext) for ext in video_extensions)


def is_image_file(file_name):
    photo_extensions = ['.jpg', '.jpeg', '.png', '.bmp', '.webp']  # 添加其他照片文件格式
    # 将文件名后缀转换为小写，并检查是否在照片文件后缀列表中
    return any(file_name.lower().endswith(ext) for ext in photo_extensions)


def is_document_file(file_name):
    document_extensions = ['.doc', '.docx', '.pdf', '.txt', '.odt']  # 添加其他文档文件格式

    # 将文件名后缀转换为小写，并检查是否在文档文件后缀列表中
    return any(file_name.lower().endswith(ext) for ext in document_extensions)


def o_d():
    return datetime.now().strftime("%Y%m%d-%H%M%S")


def mkdir_or_exist(dir_name, mode=0o777):
    if dir_name == '':
        return
    dir_name = os.path.expanduser(dir_name)
    os.makedirs(dir_name, mode=mode, exist_ok=True)


def append_current_dir(filepath):
    current_dir = os.path.dirname(os.path.abspath(filepath))
    sys.path.append(current_dir)
    return current_dir


def append_parent_dir(filepath):
    current_dir = os.path.dirname(os.path.abspath(filepath))
    parent_dir = os.path.dirname(current_dir)
    sys.path.append(parent_dir)
    return parent_dir


def load_json(path):
    with open(path, "r") as f:
        return json.load(f)
    
    
def dump_json(d, path):
    with open(path, "w") as f:
        return json.dump(d, f, indent=4)


def pt2numpy(images):
    images.cpu().permute(0, 2, 3, 1).float().numpy()
    return images


def figure2pil(fig):
    buf = io.BytesIO()
    fig.savefig(buf, bbox_inches="tight", pad_inches=0.1)
    buf.seek(0)
    return PIL.Image.open(buf)


def normalize_image(image: np.ndarray, target_min: float = 0.0, target_max: float = 1.0) -> np.ndarray:
    """
    Normalize the pixel values of an image to a specified range.

    Parameters:
        image (np.ndarray): The input image as a NumPy array.
        target_min (float): The minimum value of the normalized output range. Default is 0.0.
        target_max (float): The maximum value of the normalized output range. Default is 1.0.

    Returns:
        np.ndarray: The normalized image with pixel values in the specified range.
    """
    min_val = np.min(image)
    max_val = np.max(image)

    if max_val == min_val:
        raise ValueError("Maximum and minimum values of the image are the same. Normalization cannot be performed.")

    normalized_image = (image - min_val) / (max_val - min_val) * (target_max - target_min) + target_min
    return normalized_image

#TODO
def use_importlib():
    pass